;;;-*-Mode: Lisp; Package: MODALYS -*-

;;;----------------------------------------------------------------------
;;; Modalys, example 5.
;;;----------------------------------------------------------------------
;;; injects a simple FM synthesis signal into a plate

(new)
(set-precision 'float)
;;;
;;; setq fm synthesis specifications
;;;

;;; carrier-freq
(defvar cf) 
(setq cf (const 110))
;;; mod-index
(defvar mi) 
(setq mi (const 1.7))
;;; mod-ratio
(defvar mr)
(setq mr (const 3.2))
;;; amp-env
(defvar ae)
(setq ae (make-controller 'envelope 1 (list (list 0.00  0.0)
                                                 (list 0.10  1.0)
                                                 (list 0.70  0.8)
                                                 (list 1.20  0.0))))
;;;
;;; assemble the network
;;;

(defun +~ (ctlr0 ctlr1) (make-controller 'arithmetic 1 '+ (list ctlr0 ctlr1)))
(defun *~ (ctlr0 ctlr1) (make-controller 'arithmetic 1 '* (list ctlr0 ctlr1)))
(defun sin~ (frequency) (make-controller 'sine 1 frequency))
;;; we assume here that cf, mr, mi and ae are bound to one-dimensional controllers
(setq mrcf (*~ mr cf))
(setq fm-output (*~ (sin~ (+~ (*~ (*~ mrcf mi) (sin~ mrcf)) cf)) ae))
;(setq fm-output (*~ (sin~ (+~ (*~ mrcf mi (sin~ mrcf)) cf)) ae))


;;;
;;; vibrate a free circular plate with this signal
;;;

(defvar my-plate)
	(setq my-plate
		(with-compiled-accesses (list (list .2 0)
                                 (list .3 30))
     (make-object 'free-circ-plate (modes 1000)
                                   (radius 1)
                                   (thickness .002)
                                   (density 10000)
                                   (freq-loss .05)
                                   (const-loss .05))))

(defvar my-plate-exc)
(setq my-plate-exc (make-access my-plate (const 1 0) 'decoded))
(make-connection 'force my-plate-exc fm-output)

;;;
;;; listen to another location on the plate body
;;;

(defvar my-plate-out)
(setq my-plate-out (make-access my-plate (const 0 1) 'decoded))
(make-point-output my-plate-out)

;;;
;;; make samples and play sound
;;;

(run 5)
(play)
